unit u_CreateList;
{
  Form used for selecting/generating the list of files to
  MD5sum.
}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Buttons, ExtCtrls, u_AncestorForm, ImgList;

type
  TInfoType = (itDirectory, itFile);

  TFileinfo = class(TObject)
  public
    FullPath, DisplayName: String;
    Icon: Integer;
    InfoType: TInfoType;
  end;

  TCreateList = class(TAncestorForm)
    FoldersLabel: TLabel;
    RecurseButton: TButton;
    NoRecurseButton: TButton;
    ClearButton: TButton;
    NumItemsLabel: TLabel;
    MainPanel: TPanel;
    List: TListBox;
    Splitter: TSplitter;
    FileListBox: TListBox;
    ImageList1: TImageList;
    SelAllButton: TButton;
    OKButton: TButton;
    Button1: TButton;
    procedure RecurseButtonClick(Sender: TObject);
    procedure NoRecurseButtonClick(Sender: TObject);
    procedure ClearButtonClick(Sender: TObject);
    procedure SelAllBitBtnClick(Sender: TObject);
    procedure FileListBoxDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure FileListBoxDblClick(Sender: TObject);
    procedure SplitterMoved(Sender: TObject);
  private
    RootFolder: String;
    procedure LoadFolder(Folder: String);
  protected
    function  GetFileStringList: TStrings;
  public
    constructor Create(AOwner: TComponent; Root: String); reintroduce;
    destructor Destroy; override;
    function Excecute: Boolean;
    property FileStringList: TStrings read GetFileStringList;
  end;

function FileInfoCompare(Item1, Item2: Pointer): Integer;

var
  CreateList: TCreateList;

implementation

{$R *.DFM}

uses u_DoMD5s, Misc, ShellAPI;

{ TFileinfo }

{ TCreateList }

function GetIconIndex(FileName: String): Integer;
var
  SHFileInfo: TSHFileInfo;
  Flags: Integer;
begin
  FillChar(SHFileInfo, SizeOf(SHFileInfo), #0);
  Flags := SHGFI_SYSICONINDEX or SHGFI_ICON or SHGFI_SMALLICON;

  if SHGetFileInfo(PChar(FileName), 0, SHFileInfo, SizeOf(SHFileInfo),
    Flags) <> 0 then
    Result := SHFileInfo.iIcon
  else
    Result := 0;
end;

constructor TCreateList.Create(AOwner: TComponent; Root: String);
var
  ImageListHandle: THandle;
  SHFileInfo: TSHFileInfo;
begin
  if (not FileExists(Root)) and (not DirectoryExists(Root)) then
    raise Exception.Create('Root (' + Root + ') doesn''t exist!');

  Inherited Create(AOwner);

  FillChar(SHFileInfo, SizeOf(SHFileInfo), #0);
  ImageListHandle := SHGetFileInfo('C:\', 0, SHFileInfo, SizeOf(SHFileInfo),
    SHGFI_SYSICONINDEX or SHGFI_SMALLICON);
  ImageList1.Handle := ImageListHandle;

  RootFolder := Root;
  LoadFolder(RootFolder);
end;

destructor TCreateList.Destroy;
var
  I: Integer;
begin
  for I := 0 to FileListBox.Items.Count - 1 do
  begin
    TObject(FileListBox.Items.Objects[I]).Free;
  end;

  inherited;
end;

(**** Events ****)

procedure TCreateList.RecurseButtonClick(Sender: TObject);
{ Calls GetFiles for each item selected with recurse switched on. }
var
  I: Integer;
begin
  for I := 0 to FileListBox.Items.Count - 1 do
    if FileListBox.Selected[I] then
      GetFiles(List.Items, TFileInfo(FileListBox.items.Objects[I]).FullPath,
        '*.*', True);

  NumItemsLabel.Caption := 'Number of Items: ' + inttostr(List.Items.Count);

  OKButton.Enabled := List.Items.Count > 0;
end;

procedure TCreateList.NoRecurseButtonClick(Sender: TObject);
{ Calls GetFiles for each item selected with recurse switched off. }
var
  I: Integer;
begin
  for I := 0 to FileListBox.Items.Count - 1 do
    if FileListBox.Selected[I] then
      GetFiles(List.Items, TFileInfo(FileListBox.items.Objects[I]).FullPath,
        '*.*', False);

  NumItemsLabel.Caption := 'Number of Items: ' + inttostr(List.Items.Count);

  OKButton.Enabled := List.Items.Count > 0;
end;

procedure TCreateList.ClearButtonClick(Sender: TObject);
{ Clears the list of files. }
begin
  List.Clear;
  NumItemsLabel.Caption := 'Number of Items: ' + inttostr(List.Items.Count);

  OKButton.Enabled := List.Items.Count > 0;
end;

procedure TCreateList.SelAllBitBtnClick(Sender: TObject);
{ Selects all files and folders in the ListView. }
var
  I, TopIdx: Integer;
begin
  TopIdx := FileListBox.TopIndex;

  for I := 0 to FileListBox.Items.Count - 1 do
    FileListBox.Selected[I] := True;

  // Have to SetFocus or they don't stay selected.
  FileListBox.SetFocus;

  OKButton.Enabled := List.Items.Count > 0;

  FileListBox.TopIndex := TopIdx;
end;

function TCreateList.Excecute: Boolean;
begin
  Result := (ShowModal = mrOk);
end;

function TCreateList.GetFileStringList: TStrings;
begin
  Result := List.Items;
end;

procedure TCreateList.FileListBoxDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  FileInfo: TFileinfo;
  DestRect, SourceRect: TRect;
  Bitmap: TBitmap;
begin
  Bitmap := Nil;
  try
    try
      Bitmap := TBitmap.Create;

      FileInfo := TFileInfo(FileListBox.Items.Objects[Index]);

      if odSelected in State then
        ImageList1.DrawingStyle := dsFocus
      else
        ImageList1.DrawingStyle := dsNormal;

      ImageList1.GetBitmap(FileInfo.Icon, Bitmap);

      DestRect.Left := Rect.Left;
      DestRect.Top := Rect.Top;
      DestRect.Right := Rect.Left + 16;
      DestRect.Bottom := Rect.Top + 16;
      SourceRect.Left := 0;
      SourceRect.Top := 0;
      SourceRect.Right := 16;
      SourceRect.Bottom := 16;

      FileListBox.Canvas.FillRect(Rect);
      FileListBox.Canvas.TextRect(Rect, DestRect.Right + 2, DestRect.Top + 2,
        FileInfo.DisplayName);
      FileListBox.Canvas.BrushCopy(DestRect, Bitmap, SourceRect, clWhite);
    except
      on E: Exception do ShowMessage(E.Message);
    end;
  finally
    Bitmap.Free;
  end;
end;

function FileInfoCompare(Item1, Item2: Pointer): Integer;
var
  FileInfo1, FileInfo2: TFileInfo;
begin
  FileInfo1 := Item1;
  FileInfo2 := Item2;
  if (FileInfo1.InfoType = FileInfo2.InfoType) then
    Result := CompareStr(LowerCase(FileInfo1.DisplayName),
      LowerCase(FileInfo2.DisplayName))
  else if (FileInfo1.InfoType = itDirectory) and (FileInfo2.InfoType = itFile) then
    Result := -1
  else
    Result := 1;
end;

procedure TCreateList.LoadFolder(Folder: String);
var
  StringList: TStringList;
  I, LastSlash: Integer;
  FileInfo: TFileinfo;
  TempList: TList;
  UpPath: String;
begin
  for I := 0 to FileListBox.Items.Count - 1 do
  begin
    TObject(FileListBox.Items.Objects[I]).Free;
  end;
  FileListBox.Clear;

  TempList := TList.Create;

  // Get icon info.
  StringList := TStringList.Create;
  GetFiles(StringList, Folder, '*.*', False, True);
  for I := 0 to StringList.Count - 1 do
  begin
    FileInfo := TFileinfo.Create;
    FileInfo.FullPath := StringList[I];
    FileInfo.DisplayName := ExtractFileName(StringList[I]);
    TempList.Add(FileInfo);
    if DirectoryExists(StringList[I]) then
      FileInfo.InfoType := itDirectory
    else
      FileInfo.InfoType := itFile;
    FileInfo.Icon := GetIconIndex(StringList[I]);
  end;
  StringList.Free;

  // Sort
  TempList.Sort(FileInfoCompare);

  UpPath := Folder;
  if UpPath[Length(UpPath)] = '\' then
    Delete(UpPath, Length(UpPath), 1);
  LastSlash := 0;
  I := Pos('\', UpPath);
  while I > 0 do
  begin
    LastSlash := LastSlash + I;
    Delete(UpPath, 1, I);
    I := Pos('\', UpPath);
  end;
  UpPath := copy(Folder, 1, LastSlash - 1);

  if DirectoryExists(UpPath) and (Folder <> RootFolder) then
  begin
    FileInfo := TFileinfo.Create;
    FileInfo.FullPath := UpPath;
    FileInfo.DisplayName := '..';
    FileInfo.Icon := GetIconIndex('..');
    FileInfo.InfoType := itDirectory;
    TempList.Insert(0, FileInfo);
  end;

  for I := 0 to TempList.Count - 1 do
    FileListBox.Items.AddObject(TFileinfo(TempList[I]).DisplayName,
      TempList[I]);

  OKButton.Enabled := List.Items.Count > 0;

  TempList.Free;
end;

procedure TCreateList.FileListBoxDblClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to FileListBox.Items.Count - 1 do
    if FileListBox.Selected[I] then
      if DirectoryExists(TFileInfo(FileListBox.Items.Objects[I]).FullPath) then
      begin
        LoadFolder(TFileInfo(FileListBox.Items.Objects[I]).FullPath);
        Break;
      end
      else
      if FileExists(TFileInfo(FileListBox.Items.Objects[I]).FullPath) then
      begin
        List.Items.Add(TFileInfo(FileListBox.Items.Objects[I]).FullPath);
      end;

  OKButton.Enabled := List.Items.Count > 0;
end;

procedure TCreateList.SplitterMoved(Sender: TObject);
begin
  NumItemsLabel.Left := List.Left + 4 + 8; // 4 to ;ine up, 8 offset.
end;

end.
